#include <CapacitiveSensor.h>

#define C1_note 130.81
#define D1_note 146.83
#define E1_note 164.81
#define F1_note 174.61
#define G1_note 196.00
#define A1_note 220.00
#define B1_note 246.94
#define C2_note 261.63
#define D2_note 293.66

const int loopEnable = A0; // loop BUTTON pin
const int leadEnable = A5; // lead BUTTON pin
const int rythmEnable = A4; // rythm BUTTON pin

const int rythmPin =  A2; // rythm LED pin
const int leadPin =  A3; // lead LED pin
const int loopPin =  A1; // loop LED pin

int leadState = 0;
int rythmState = 0;
int loopState = 0;

int lastLoopState = 0;
int lastLeadState = 0;
int lastRythmState = 0;
int loopCount = 0;

const int maxNotes = 10;

int loopNotesLead[maxNotes];
int loopNotesRythm[maxNotes];

int crtNoteLead = 0;
int crtNoteRythm = 0;

bool recordNotes = false;
bool playNotes = false;

int buzzer_crt = 0;
int leadBuzzer = 5;
int rythmBuzzer = 13;

CapacitiveSensor   cs_4_2 = CapacitiveSensor(4,2); // 10M resistor between pins 4 and 2; pin 2 = sensor pin
CapacitiveSensor   cs_4_3 = CapacitiveSensor(4,3); // 10M resistor between pins 4 and 3; pin 3 = sensor pin
CapacitiveSensor   cs_4_6 = CapacitiveSensor(4,6); // 10M resistor between pins 4 and 6; pin 6 = sensor pin

CapacitiveSensor   cs_4_7 = CapacitiveSensor(4,7); // 10M resistor between pins 4 and 7; pin 7 = sensor pin
CapacitiveSensor   cs_4_8 = CapacitiveSensor(4,8); // 10M resistor between pins 4 and 8; pin 8 = sensor pin
CapacitiveSensor   cs_4_9 = CapacitiveSensor(4,9); // 10M resistor between pins 4 and 9; pin 9 = sensor pin

CapacitiveSensor   cs_4_10 = CapacitiveSensor(4,10); // 10M resistor between pins 4 and 10; pin 10 = sensor pin
CapacitiveSensor   cs_4_11 = CapacitiveSensor(4,11); // 10M resistor between pins 4 and 11; pin 11 = sensor pin
CapacitiveSensor   cs_4_12 = CapacitiveSensor(4,12); // 10M resistor between pins 4 and 12; pin 12 = sensor pin

void setup() {
  pinMode(loopPin, OUTPUT);
  pinMode(loopEnable, INPUT);

  pinMode(leadPin, OUTPUT);
  pinMode(leadEnable, INPUT);

  pinMode(rythmPin, OUTPUT);
  pinMode(rythmEnable, INPUT);

  Serial.begin(9600);
}

void loop(){
  leadState = digitalRead(leadEnable);
  rythmState = digitalRead(rythmEnable);

  // Check if lead BUTTON is pressed
  if(leadState != lastLeadState){
    if(leadState == HIGH){
      crtNoteLead = 0;
      digitalWrite(leadPin, HIGH);
      digitalWrite(rythmPin, LOW);
      buzzer_crt = 1;
    }
  }
  
  // Check if rythm BUTTON is pressed
  if(rythmState != lastRythmState){
    if(rythmState == HIGH){
      crtNoteRythm = 0;
      digitalWrite(leadPin, LOW);
      digitalWrite(rythmPin, HIGH);
      buzzer_crt = 0;
    }
  }
  lastLeadState = leadState;
  lastRythmState = rythmState;
  
  loopState = digitalRead(loopEnable);
  
  // Check if the loop BUTTON state has changed
  if(loopState != lastLoopState){
    if(loopState == HIGH){
      loopCount++;
      // First time pushing the loop button -> Enable loop function. Start Recording(loop LED on)
      if(loopCount == 1){
        digitalWrite(loopPin, HIGH);
        Serial.print("loop ON. Start recording");
        recordNotes = true;
      }
      // Second time pushing the loop button -> Stop Recording. Play notes (loop LED on)
      if(loopCount == 2 && playNotes == false){
        playNotes = true;
        recordNotes = false;
        digitalWrite(loopPin, HIGH);
        Serial.print("stop recording");
      }
      // Third time pushing the loop button -> Disable loop function(loop LED off)
      if(loopCount == 3){
        playNotes = false;
        digitalWrite(loopPin, LOW);
        Serial.print("loop OFF");
        loopCount = 0;
        crtNoteLead = 0;
        crtNoteRythm = 0;
      }
    }  
    delay(100);
  }
  lastLoopState = loopState;

  // Playing recorded notes.
  if(playNotes == true){
      if(buzzer_crt == 1){
          for(int i = 0; i < crtNoteLead; i++){
            tone(leadBuzzer, loopNotesLead[i]);
            delay(200);
          }
      }
      else{
          for(int i = 0; i < crtNoteRythm; i++){
            tone(rythmBuzzer, loopNotesRythm[i]);
            delay(200);
          }
      }
  }
  
  long touch1_3 =  cs_4_2.capacitiveSensor(30); // E1
  long touch2_3 =  cs_4_3.capacitiveSensor(30); // A1
  long touch3_3 =  cs_4_6.capacitiveSensor(30); // D2
  long touch1_2 =  cs_4_7.capacitiveSensor(30); // D1
  long touch2_2 =  cs_4_8.capacitiveSensor(30); // G1
  long touch3_2 =  cs_4_9.capacitiveSensor(30); // C2
  long touch1_1 =  cs_4_10.capacitiveSensor(30); // C1
  long touch2_1 =  cs_4_11.capacitiveSensor(30); // F1
  long touch3_1 =  cs_4_12.capacitiveSensor(30); // B1

  if(touch1_3>1000){
    play_note(E1_note, buzzer_crt);
  }
  else if(touch2_3>1000){
    play_note(A1_note, buzzer_crt);
  }
  else if(touch3_3>1000){
    play_note(D2_note, buzzer_crt);
  }
  else if(touch1_2>1000){
    play_note(D1_note, buzzer_crt);
  }
  else if(touch2_2>1000){
    play_note(G1_note, buzzer_crt);
  }
  else if(touch3_2>1000){
    play_note(C2_note, buzzer_crt);
  }

  else if(touch1_1>1000){
    play_note(C1_note, buzzer_crt);
  }
  else if(touch2_1>1000){
    play_note(F1_note, buzzer_crt);
  }
  else if(touch3_1>1000){
    play_note(B1_note, buzzer_crt);
  }
  else{
    if(buzzer_crt == 1)
      noTone(leadBuzzer);
     else
      noTone(rythmBuzzer);
  }

  Serial.print("\t");
  delay(100);
}

void play_note(double note, int buzzer_crt){
    // Note should be playes on lead
    if(buzzer_crt == 1){
      tone(leadBuzzer, note);
      delay(10);
      // If recording is enabled -> add current note
      if(recordNotes == true){
        loopNotesLead[crtNoteLead] = note;
        crtNoteLead++;
      }
    }
    // Note should be playes on rythm
    else{
      tone(rythmBuzzer, note);
      delay(200);
      // If recording is enabled -> add current note
      if(recordNotes == true){
        loopNotesRythm[crtNoteRythm] = note;
        crtNoteRythm++;
      }
    }   
}
